#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var commander_1 = require("commander");
var global_1 = require("../lib/core/global");
var bst_virtual_alexa_1 = require("../lib/client/bst-virtual-alexa");
var logging_helper_1 = require("../lib/core/logging-helper");
var chalk = require("chalk");
commander_1.program.version(global_1.Global.version());
commander_1.program
    .usage("[options]")
    .option("-u, --url <alexa-skill-url>", "The URL of the Alexa skill to speak to - defaults to current proxied skill")
    .option("-l, --locale <locale>", "The locale expected for the Alexa skill - defaults to en-US")
    .option("-m, --model <interaction-model-path>", "Path to the interaction model file - defaults to ./models/en-US.json")
    .option("-i, --intents <intent-schema-path>", "Path to the intent schema file - defaults to ./speechAssets/IntentSchema.json")
    .option("-s, --samples <sample-utterances-path>", "Path to the sample utterances file - defaults to ./speechAssets/SampleUtterances.txt")
    .option("-a, --appId <application-id>", "The application ID for the skill")
    .option("-U, --userId <user-id>", "Sets the user id to the specified value")
    .option("-t, --accessToken <accessToken>", "Sets the access token for emulating a user with a linked account")
    .description("Creates an launch request and sends it to your skill")
    .action(function () {
    var options = commander_1.program.opts();
    var url = options.url;
    var interactionModel = options.model;
    var intentSchemaPath = options.intents;
    var samplesPath = options.samples;
    var applicationID = options.appId;
    var locale = options.locale;
    var userId = options.userId;
    if (process.argv.some(function (arg) { return arg === "-h" || arg === "--help"; })) {
        commander_1.program.outputHelp();
        process.exit(0);
        return;
    }
    if (!options.url) {
        var proxyProcess = global_1.Global.running();
        if (proxyProcess === null) {
            console.error("No URL specified and no proxy is currently running");
            console.log();
            console.log("URL (--url) must be specified if no proxy is currently running");
            console.log();
            console.log("If a proxy is running, the launch request will automatically be sent to it");
            console.log();
            process.exit(0);
            return;
        }
        url = "http://localhost:" + proxyProcess.port;
    }
    var savedSession = global_1.Global.config().loadSession() || {};
    var localeToUse = locale ? locale : savedSession.locale;
    var userIdToUse = userId ? userId : savedSession.userId;
    var speaker = new bst_virtual_alexa_1.BSTVirtualAlexa(url, interactionModel, intentSchemaPath, samplesPath, applicationID, localeToUse, userIdToUse);
    try {
        speaker.start(true);
    }
    catch (error) {
        process.exit(0);
        return;
    }
    if (userIdToUse) {
        speaker.context().user().setID(userIdToUse);
    }
    if (options.accessToken) {
        speaker.context().setAccessToken(options.accessToken);
    }
    speaker.launched(function (errorInLaunch, response, request) {
        if (errorInLaunch) {
            console.error(chalk.red("Error: " + errorInLaunch));
            return;
        }
        var jsonPretty = JSON.stringify(response, null, 4);
        console.log("Request:");
        console.log(chalk.hex(logging_helper_1.LoggingHelper.REQUEST_COLOR)(JSON.stringify(request, null, 4)));
        console.log("");
        console.log("Response:");
        console.log(chalk.cyan(jsonPretty));
        console.log("");
    });
});
global_1.Global.initializeCLI().then(function () {
    process.argv.splice(2, 0, "extra");
    commander_1.program.parse(process.argv);
});
//# sourceMappingURL=bst-launch.js.map